import sys
from functools import partial

from blinker import Namespace
from cpl_core.dependency_injection import ServiceProviderABC
from flask import Flask, request
from flask_cors import CORS

from bot_api.configuration.api_settings import ApiSettings
from bot_api.logging.api_logger import ApiLogger
from bot_api.route.route import Route


class Api(Flask):

    def __init__(
            self,
            logger: ApiLogger,
            services: ServiceProviderABC,
            api_settings: ApiSettings,
            *args, **kwargs
    ):
        if not args:
            kwargs.setdefault('import_name', __name__)

        Flask.__init__(self, *args, **kwargs)

        self._logger = logger
        self._services = services
        self._apt_settings = api_settings

        self._cors = CORS(self, support_credentials=True)

        # register before request
        self.before_request_funcs.setdefault(None, []).append(self.before_request)
        my_signals = Namespace()
        notify = my_signals.signal('notify')

    def _register_routes(self):
        for path, f in Route.registered_routes.items():
            route = f[0]
            kwargs = f[1]
            cls = None
            qual_name_split = route.__qualname__.split('.')
            if len(qual_name_split) > 0:
                cls_type = vars(sys.modules[route.__module__])[qual_name_split[0]]
                cls = self._services.get_service(cls_type)

            partial_f = partial(route, self if cls is None else cls)
            partial_f.__name__ = route.__name__
            self.route(path, **kwargs)(partial_f)

    def before_request(self, *args, **kwargs):
        self._logger.debug(__name__, f'Received GET @{request.url}')
        headers = str(request.headers).replace("\n", "\n\t")
        self._logger.trace(__name__, f'Headers: \n\t{headers}')

    def start(self):
        self._logger.info(__name__, f'Starting API {self._apt_settings.host}:{self._apt_settings.port}')
        self._register_routes()
        from waitress import serve
        serve(self, host=self._apt_settings.host, port=self._apt_settings.port)
