import traceback
from typing import Optional

from cpl_core.console import Console

from bot_api.abc.dto_abc import DtoABC
from bot_data.model.auth_role_enum import AuthRoleEnum


class AuthUserDTO(DtoABC):

    def __init__(
            self,
            id: int,
            first_name: str,
            last_name: str,
            e_mail: str,
            password: str,
            confirmation_id: Optional[str],
            auth_role: AuthRoleEnum,
    ):
        DtoABC.__init__(self)

        self._id = id
        self._first_name = first_name
        self._last_name = last_name
        self._email = e_mail
        self._password = password
        self._is_confirmed = confirmation_id is None
        self._auth_role = auth_role
        
    @property
    def id(self) -> int:
        return self._id

    @property
    def first_name(self) -> str:
        return self._first_name

    @first_name.setter
    def first_name(self, value: str):
        self._first_name = value

    @property
    def last_name(self) -> str:
        return self._last_name

    @last_name.setter
    def last_name(self, value: str):
        self._last_name = value

    @property
    def email(self) -> str:
        return self._email

    @email.setter
    def email(self, value: str):
        self._email = value

    @property
    def password(self) -> str:
        return self._password

    @password.setter
    def password(self, value: str):
        self._password = value

    @property
    def is_confirmed(self) -> Optional[str]:
        return self._is_confirmed

    @is_confirmed.setter
    def is_confirmed(self, value: Optional[str]):
        self._is_confirmed = value

    @property
    def auth_role(self) -> AuthRoleEnum:
        return self.auth_role

    @auth_role.setter
    def auth_role(self, value: AuthRoleEnum):
        self.auth_role = value

    def from_dict(self, values: dict):
        self._id = values['Id']
        self._first_name = values['FirstName']
        self._last_name = values['LastName']
        self._email = values['EMail']
        self._password = values['Password']
        self._is_confirmed = values['IsConfirmed']
        self._auth_role = values['AuthRole']

    def to_dict(self) -> dict:
        return {
            "Id": self._id,
            "FirstName": self._first_name,
            "LastName": self._last_name,
            "EMail": self._email,
            "Password": self._password,
            "IsConfirmed": self._is_confirmed,
            "AuthRole": self._auth_role,
        }
